# coding: utf-8
import re, codecs, sys

#
#	フォント変換表
#
font_conv_table = [	( r"\\mathbb\{N\}", u"ℕ" ),	# \mathbb
			( r"\\mathbb\{Z\}", u"ℤ" ),
			( r"\\mathbb\{Q\}", u"ℚ" ),
			( r"\\mathbb\{R\}", u"ℝ" ),
			( r"\\mathbb\{C\}", u"ℂ" ),

			( r"\\cdots", u"⋯" ),		# \cdots
			( r"\\cdot", u"⋅" ),
			( r"\\times", u"×" ),
			( r"\\sum", u"∑" ),
			( r"\\prod", u"∏" ),
			( r"\\infty", u"∞" ),
			( r"\\mid", r"|" ),

			( r"\\in[\s]+", u"∈" ),	# 集合
			( r"\\nin[\s]+", u"∉" ),
			( r"\\ni[\s]+", u"∋" ),
			( r"\\nni[\s]+", u"∌" ),
			( r"\\emptyset", u"∅" ),
			( r"\\subsetneqq", u"⊊" ),
			( r"\\subsetneq", u"⊊" ),
			( r"\\subseteqq", u"⊆" ),
			( r"\\subset", u"⊂" ),
			( r"\\supsetneqq", u"⊋" ),
			( r"\\supsetneq", u"⊋" ),
			( r"\\supseteqq", u"⊇" ),
			( r"\\supseteq", u"⊇" ),
			( r"\\supset", u"⊃" ),
			( r"\\cup[\s]+", u"∪" ),
			( r"\\cap[\s]+", u"∩" ),

			( r"\\geqq", u"≧" ),		# 等号・不等号
			( r"\\leqq", u"≦" ),
			( r"\\neqq", u"≠" ),

			( r"\\partial", u"∂" ),	# 微分
			( r"\\sim[\s]+", u"∽" ),
			( r"\\approx", u"≈" ),

			( r"\\alpha", u"α" ),		# ギリシャ文字
			( r"\\beta", u"β" ),
			( r"\\gamma", u"γ" ),
			( r"\\delta", u"δ" ),
			( r"\\varepsilon", u"ℇ" ),

			( r"\\Rightarrow", u"⇒" ),	# 矢印
			( r"\\Leftarrow", u"⇐" ),

			( r"\\\{", r"{" ),		# 特殊文字
			( r"\\\}", r"}" ),

			( r"\\bunsuu", r"\\frac" ),

			( r"\\quad", "" ),		# 消去
			( r"\\label\{.*\}", "" ),
			( r"\\vspace\*\{.*\}", "" ),
			( r"\\begin\{breakbox\}", "" ),
			( r"\\end\{breakbox\}", "" )	]

#
#	コマンド行引数検査
#
if len( sys.argv ) != 2:
	print u"convert.py：シフトJIS LaTeXソースをFormedTextへ変換します。"
	print u"使い方： convert.py [texファイル名(.tex)]"
	sys.exit(0)

#
#	ファイル名検査とファイルオープン
#
src = sys.argv[1]
if re.search( r"\.tex$", src ) == None:
	print u"convert.py：シフトJIS LaTeXソースをFormedTextへ変換します。"
	print u"使い方： convert.py [texファイル名(.tex)]"
	sys.exit(0)
dst = re.sub( r"\.tex$", ".txt", src )
print u"変換元ファイル名:", src
print u"変換先ファイル名:", dst
try:
	fin  = codecs.open( src, "r", "shift_jis" )
	fout = codecs.open( dst, "w", "utf_8" )
except IOError:
	print u"ファイルオープンエラーです。"
	sys.exit(0)

#
#	LaTeX から FormedText への変換
#

# ファイルから読み込み
str = unicode( fin.read() )

#
#	フォント変換
#
for pair in font_conv_table:
	str = re.sub( pair[0], pair[1], str )

#
#	数式モード・数式環境
#

# $...$ --> <qt>...</qt>
str = re.sub( r"\$([^$]+)\$", r"<qt>\1</qt>", str )

# \[...\] --> <qd>...</qd>
str = re.sub( r"\\\[", "<qd>", str )
str = re.sub( r"\\\]", "</qd>", str )

# equation --> eqn
str = re.sub( r"\\begin\{equation\}", "<eqn>", str )
str = re.sub( r"\\end\{equation\}", "</eqn>", str )

# eqnarray* --> qdarray
str = re.sub( r"\\begin\{eqnarray\*\}", "<qdarray>", str )
str = re.sub( r"\\end\{eqnarray\*\}", "</qdarray>", str )

# eqnarray --> eqnarray
str = re.sub( r"\\begin\{eqnarray\}", "<eqnarray>", str )
str = re.sub( r"\\end\{eqnarray\}", "</eqnarray>", str )

# &.&
str = re.sub( r"&(.)&", r"{\1}", str )

#
#	ルビ
#

# \ruby{\1}{\2} --> <rem>\1:\2</rem>
str = re.sub( r"\\textgt\{([^}]*)\}", r"\1", str )
str = re.sub( r"\\ruby[\s]*\{([^}]*)\}[\s]*\{([^}]*)\}", r"<rem>\1:\2</rem>", str )

#
#	ブロック構造
#

# \section{...} --> ****...
str = re.sub( r"\\section\{([^}]*)\}", r"****\1", str )

# \subsection{...} --> ====...
str = re.sub( r"\\subsection\{([^}]*)\}", r"====\1", str )

# \subsubsection*{...} --> ----...
str = re.sub( r"\\subsubsection\*\{([^}]*)\}", r"----\1", str )

# proposition
str = re.sub( r"\\begin\{proposition\}\[([^\]]*)\]", u"++++命題:" + r"\1", str )
str = re.sub( r"\\end\{proposition\}", "\n", str )
str = re.sub( r"\\begin\{proposition\}", u"++++命題", str )
str = re.sub( r"\\end\{proposition\}", "\n", str )

# theorem
str = re.sub( r"\\begin\{theorem\}\[([^\]]*)\]", u"++++定理:" + r"\1", str )
str = re.sub( r"\\end\{theorem\}", "\n", str )
str = re.sub( r"\\begin\{theorem\}", u"++++定理", str )
str = re.sub( r"\\end\{theorem\}", "\n", str )

# axiom
str = re.sub( r"\\begin\{axiom\}\[([^\]]*)\]", u"++++公理:" + r"\1", str )
str = re.sub( r"\\end\{axiom\}", "\n", str )
str = re.sub( r"\\begin\{axiom\}", u"++++公理", str )
str = re.sub( r"\\end\{axiom\}", "\n", str )

# definition
str = re.sub( r"\\begin\{definition\}\[([^\]]*)\]", u"++++定義:" + r"\1", str )
str = re.sub( r"\\end\{definition\}", "\n", str )
str = re.sub( r"\\begin\{definition\}", u"++++定義", str )
str = re.sub( r"\\end\{definition\}", "\n", str )

# lemma
str = re.sub( r"\\begin\{lemma\}\[([^\]]*)\]", u"++++補題:" + r"\1", str )
str = re.sub( r"\\end\{lemma\}", "\n", str )
str = re.sub( r"\\begin\{lemma\}", u"++++補題", str )
str = re.sub( r"\\end\{lemma\}", "\n", str )

# corollary
str = re.sub( r"\\begin\{corollary\}\[([^\]]*)\]", u"++++系:" + r"\1", str )
str = re.sub( r"\\end\{corollary\}", "\n", str )
str = re.sub( r"\\begin\{corollary\}", u"++++系", str )
str = re.sub( r"\\end\{corollary\}", "\n", str )

# example
str = re.sub( r"\\begin\{example\}\[([^\]]*)\]", u"++++例題:" + r"\1", str )
str = re.sub( r"\\end\{example\}", "\n", str )
str = re.sub( r"\\begin\{example\}", u"++++例題", str )
str = re.sub( r"\\end\{example\}", "\n", str )

# algorithm
str = re.sub( r"\\begin\{algorithm\}\[([^\]]*)\]", u"++++算法:" + r"\1", str )
str = re.sub( r"\\end\{algorithm\}", "\n", str )
str = re.sub( r"\\begin\{algorithm\}", u"++++算法", str )
str = re.sub( r"\\end\{algorithm\}", "\n", str )

# proof
str = re.sub( r"\\begin\{proof\}\[([^\]]*)\]", r"////\1", str )
str = re.sub( r"\\end\{proof\}", "\n", str )
str = re.sub( r"\\begin\{proof\}", u"////証明", str )
str = re.sub( r"\\end\{proof\}", "\n", str )

#
#	プログラムリスト
#

# verbatim --> pre
str = re.sub( r"\\begin\{verbatim\}", "<pre>", str )
str = re.sub( r"\\end\{verbatim\}", "</pre>", str )

# \begin{filelist}{...} --> ----...
str = re.sub( r"\\begin\{filelist\}\{([^}]*)\}", r"----\1", str )
str = re.sub( r"\\end\{filelist\}", "\n", str )

#
#	箇条書き
#

# description --> qlist
str = re.sub( r"\\begin\{description\}", "<qlist left='15'>", str )
str = re.sub( r"\\end\{description\}", "</qlist>", str )

# \item[...] --> ...:
str = re.sub( r"\\item\[([^\]]*)\]", r"\1:", str )

# ファイルへ書き込み
fout.write( str )
